import datetime
import random
import time

import pandas as pd

COLUMN_NAMES = ["date", "flow_start", "duration", "proto", "src", "blank",
                "dst", "flags", "tos", "packets", "bytes", "flows"]

ROWS_TO_SKIP = [0, 1]


class DataLoader(object):

    def __init__(self, input_data):
        self.total_size = 0
        self.input = input_data
        self.session_start = None
        self.frame = None

    def clean_data(self, df):
        df["timestamp"] = df["date"] + " " + df["flow_start"]
        df["timestamp"] = pd.to_datetime(df["timestamp"])
        df = df.sort_values("timestamp")

        # Random 5 minutes interval select
        last_timestamp = df.iloc[-1]['timestamp']
        df['time_left'] = last_timestamp - df['timestamp']
        time_frame = df[df['time_left'] >= datetime.timedelta(minutes=5)].reset_index()
        self.session_start = df.iloc[random.randint(0, len(time_frame))]['timestamp']
        df = df[(self.session_start <= df['timestamp']) & (
                    df['timestamp'] <= self.session_start + datetime.timedelta(minutes=5))]
        df = df.drop(columns=['time_left', "blank", "flags", "tos", "flows", "date", "flow_start"], axis=1)

        return df

    def load_data_from_file(self, file):
        print("Reading file: " + file)
        start = time.time()
        df = pd.read_csv(file,
                         names=COLUMN_NAMES,
                         skiprows=ROWS_TO_SKIP,
                         delim_whitespace=True)

        df['src_ip'], df['src_port'] = zip(*df['src'].map(lambda x: x.split(":")))
        df['dst_ip'], df['dst_port'] = zip(*df['dst'].map(lambda x: x.split(":")))

        df = self.clean_data(df)
        print("Data load: " + str(time.time() - start))
        return df

    def load_data_from_list(self, input_list):
        df = pd.DataFrame(input_list)
        df = self.clean_data(df)
        self.frame = df
        return df

    def load_data_to_dataframe(self):
        if self.frame is None:
            df = None
            if type(self.input) is str:
                df = self.load_data_from_file(self.input)
            elif type(self.input) is list:
                df = self.load_data_from_list(self.input)
            self.total_size = len(df)
            self.frame = df
        return self.frame
