import gc
import time
from os import listdir
import os
import csv

import pandas as pd

from src.resolver.PairManager import PairManager
from src.resolver.RecordManager import RecordManager
from src.resolver.Helpers import AddressManager

BLANK = "-"

POSITIVE_PATH = "../../resources/data/possitive/"
NEGATIVE_PATH = "../../resources/data/negative/"

keys = ['duration', 'proto', 'req_timestamp', 'req_src', 'req_ip', 'req_port', 'req_dst',
        'req_packets', 'req_bytes', 'resp_timestamp', 'resp_src', 'resp_ip', 'resp_port', 'resp_dst',
        'resp_packets', 'resp_bytes', 'paired', 'tunneled']


class DataMixer(object):

    def __init__(self):
        self.used_users = set()

    def start_mixing(self):
        am = AddressManager()
        negative_files = listdir(NEGATIVE_PATH)
        positive_files = listdir(POSITIVE_PATH)
        data = []

        for i in range(0, 30):
            neg = NEGATIVE_PATH + negative_files[i]
            pos = positive_files[350 * i:350 * i + 350]
            data = data + self.mix_data(pos, neg)

            mixed_users = {}
            # Create users
            for row in data:
                if am.is_my_address_local(row["req_ip"]):
                    if row["req_ip"] not in mixed_users:
                        mixed_users[row["req_ip"]] = []
                    mixed_users[row["req_ip"]].append(row)

                # Insert into files
            for user in mixed_users:
                with open("./../../../mixed_users/" + user + "_" + str(i) + ".csv", "w") as f:
                    writer = csv.DictWriter(f, keys)
                    writer.writeheader()
                    writer.writerows(mixed_users[user])

            mixed_users = {}
            data = []
            gc.collect()

    def mix_data(self, positive, negative):
        self.used_users = set()

        neg = PairManager(negative)
        negative_data = neg.load_data_to_dataframe()
        start_negative_timestamp = neg.session_start

        result_mix = []
        current_used = set()

        for positive_file in positive:
            random_row = self.get_random_negative(negative_data)
            current_used.add(random_row['src_ip'])
            pos = PairManager(POSITIVE_PATH + positive_file)
            rm = RecordManager()
            positive_data = pos.get_paired_list()
            start_positive_timestamp = pos.session_start

            for row in positive_data:
                if row["req_src"] != BLANK:
                    row = rm.label_record(row)
                    time_diff = row['req_timestamp'] - start_positive_timestamp
                    row['req_timestamp'] = start_negative_timestamp + time_diff
                    row['req_ip'] = random_row['src_ip']
                    row['req_src'] = random_row['src_ip'] + ":" + row['req_port']

                    if row['resp_src'] != BLANK:
                        row['resp_dst'] = row['req_src']
                        time_diff = row['resp_timestamp'] - start_positive_timestamp
                        row['resp_timestamp'] = start_negative_timestamp + time_diff

                result_mix.append(row)

        start = time.time()
        neg = neg.get_paired_list(current_used)
        result_mix = result_mix + neg
        print("Paired " + str(time.time() - start))
        return result_mix

    def get_random_negative(self, data):
        am = AddressManager()

        negative_frame = data.filter(items=['src', 'src_ip', 'src_port'])

        # FILTER FOR SOURCE PORT 53 -- DNS SERVER. Tilda sign says "ALL OTHER THAN 53 SOURCE PORTS WILL BE USED"
        dns_port_filter = negative_frame[negative_frame['src_port'] == "53"]
        negative_frame = negative_frame[~negative_frame['src_ip'].isin(dns_port_filter['src_ip'])]

        # FILTER FOR UNIQUE IP ADRESSES
        negative_frame_ip = pd.DataFrame()
        negative_frame_ip['src_ip'] = negative_frame['src_ip'].drop_duplicates()
        negative_frame = negative_frame.drop(columns=['src_ip'])
        negative_frame = pd.merge(negative_frame_ip, negative_frame, left_index=True, right_index=True)

        # USE ONLY LOCAL IP ADDRESSES
        negative_frame = negative_frame[
            negative_frame.apply(lambda x: am.is_my_address_local(x['src_ip']), axis=1, reduce=True)]

        # DON'T USE ALREADY USED USERS
        negative_frame = negative_frame[
            negative_frame.apply(lambda x: x['src_ip'] not in self.used_users, axis=1, reduce=True)]

        rand_row = negative_frame.sample(n=1)
        rand_row = rand_row.squeeze()

        self.used_users.add(rand_row['src_ip'])
        return rand_row


if __name__ == '__main__':
    DataMixer().start_mixing()




