import pytest

import TestLists as td
from src.resolver.PairManager import PairManager
from random import randint
from pandas._libs.tslib import Timestamp
import datetime
import os

TEST_UNIVERSAL_FILE = '../resources/test_data/nfcapd.Test_data'

TEST_DATA_1 = td.TEST_PAIRING_1
COMP_DATA_1 = td.COMP_PAIRING_1


def test_packet_is_relevant():
    dl = PairManager(TEST_UNIVERSAL_FILE)

    # Test timestamps
    new_timestamp = datetime.datetime.now()
    new = type('obj', (object,), {"src": "192.168.0.1:33", "dst": "192.168.1.1:53", "timestamp": new_timestamp, "proto": "UDP"})

    tct = datetime.datetime.now()
    to_compare_timestamp = tct - datetime.timedelta(milliseconds=1000)
    to_compare = {"req_src": "192.168.1.1:53", "req_dst": "192.168.0.1:33",
                  "req_timestamp": to_compare_timestamp,
                  "resp_src": "-", "resp_dst": "-", "resp_timestamp": "-", "paired": False, "proto": "UDP"}
    result = dl.is_packet_relevant(new, to_compare)
    assert True == result

    to_compare_timestamp = tct - datetime.timedelta(milliseconds=500)
    to_compare.update({"req_timestamp": to_compare_timestamp})
    result = dl.is_packet_relevant(new, to_compare)
    assert True == result

    rand_milli = randint(0, 1000)
    to_compare_timestamp = tct - datetime.timedelta(milliseconds=rand_milli)
    to_compare.update({"req_timestamp": to_compare_timestamp})
    result = dl.is_packet_relevant(new, to_compare)
    assert True == result

    to_compare_timestamp = tct - datetime.timedelta(seconds=10)
    to_compare.update({"req_timestamp": to_compare_timestamp})
    result = dl.is_packet_relevant(new, to_compare)
    assert False == result

    # test src-dst
    new_timestamp = datetime.datetime.now()
    new = type('obj', (object,), {"src": "192.168.0.1:53", "dst": "192.168.1.1:33", "timestamp": new_timestamp, "proto": "UDP"})

    tct = datetime.datetime.now()
    to_compare_timestamp = tct - datetime.timedelta(milliseconds=500)
    to_compare = {"req_src": "192.168.1.1:33", "req_dst": "192.168.0.1:53",
                  "req_timestamp": to_compare_timestamp,
                  "resp_src": "-", "resp_dst": "-", "resp_timestamp": "-", "paired": False, "proto": "UDP"}

    to_compare.update({"req_src": "1.1.1.1:33"})

    result = dl.is_packet_relevant(new, to_compare)
    assert False == result

    new.dst = "1.1.1.1:33"
    result = dl.is_packet_relevant(new, to_compare)
    assert True == result

    # is already paired
    to_compare.update({"src_two": "0.0.0.0:33", "dst_two": "1.2.3.4:53", "timestamp_two": to_compare_timestamp,
                       "paired": True})
    result = dl.is_packet_relevant(new, to_compare)
    assert False == result


def test_create_map_key():
    dl = PairManager(TEST_UNIVERSAL_FILE)

    # TypeError expected
    with pytest.raises(TypeError):
        dl.create_map_key(1, 2)

    with pytest.raises(TypeError):
        dl.create_map_key(1, "dst")

    with pytest.raises(TypeError):
        dl.create_map_key("src", 2)

    with pytest.raises(TypeError):
        dl.create_map_key("src", True)

    result = dl.create_map_key("192.168.0.0", "192.168.0.1")
    assert "192.168.0.0/192.168.0.1" == result


def test_pair_records():
    dl = PairManager("nfcapd.No_File")
    with pytest.raises(FileNotFoundError):
        dl.pair_records()

    def test_paired_unpaired_count(data, total):
        paired = len(data[data["paired"] == True])
        unpaired = len(data[data["paired"] == False])
        step = paired * 2 + unpaired
        assert total == step

    dl = PairManager(TEST_DATA_1)
    data = dl.get_paired_list()

    assert COMP_DATA_1 == data
    test_paired_unpaired_count(dl.get_paired_dataframe(), dl.total_size)

    # Manual added pair test
    dl = PairManager(TEST_UNIVERSAL_FILE)
    data = dl.get_paired_dataframe()

    pair_exists = ((data["req_src"] == "10.10.10.10:1010") & (data["req_dst"] == "20.20.20.20:53") &
     (data["resp_src"] == "20.20.20.20:53") & (data["resp_dst"] == "10.10.10.10:1010") &
     (data["resp_timestamp"] == Timestamp("2013-04-24 14:18:30.000")) & (data["paired"] == True)).any()

    assert True == pair_exists

    lone_one_1 = ((data["req_src"] == "-") & (data["req_dst"] == "-") &
                (data["resp_src"] == "20.20.20.20:53") & (data["resp_dst"] == "10.10.10.10:1010") &
                (data["paired"] == False)).any()
    assert True == lone_one_1

    lone_one_2 = ((data["req_src"] == "10.10.10.10:1010") & (data["req_dst"] == "20.20.20.20:53") &
                  (data["resp_src"] == "-") & (data["resp_dst"] == "-") &
                  (data["paired"] == False)).any()
    assert True == lone_one_2

    resp_timestamp_is_lower_pair = ((data["req_src"] == "10.10.10.100:1010") & (data["req_dst"] == "20.20.20.200:53") &
                (data["resp_src"] == "20.20.20.200:53") & (data["resp_dst"] == "10.10.10.100:1010") &
                (data["paired"] == True) & (data["resp_timestamp"] == Timestamp("2013-04-24 14:18:20.000"))).any()
    assert True == resp_timestamp_is_lower_pair


















